/**
 * OW2 Util
 * Copyright (C) 2008-2010 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: BasicPoolStatsFillerPoolListener.java 5502 2010-05-26 09:06:11Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.pool.impl.enhanced.impl.listener.statistics;

import org.ow2.util.pool.impl.enhanced.api.IWaitControl;
import org.ow2.util.pool.impl.enhanced.api.listener.statistics.IPoolStatsListener;
import org.ow2.util.pool.impl.enhanced.impl.listener.PoolListenerAdapter;

/**
 * @author Gael Lalire
 * @param <E> pool item type
 */
public class BasicPoolStatsFillerPoolListener<E> extends PoolListenerAdapter<E> implements
        IPoolStatsListener<E, BasicPoolStats> {

    private BasicPoolStats basicPoolStats;

    public BasicPoolStatsFillerPoolListener() {
        this.basicPoolStats = new BasicPoolStats();
    }

    /**
     * @return the basicPoolStats
     */
    protected BasicPoolStats getBasicPoolStats() {
        return basicPoolStats;
    }

    @Override
    public void getMethodCalled(final IWaitControl timeout) {
        synchronized (basicPoolStats) {
            basicPoolStats.setWaiters(basicPoolStats.getWaiters() + 1);
        }
    }

    @Override
    public void getMethodFailed(final Exception poolException, final IWaitControl timeout) {
        synchronized (basicPoolStats) {
            basicPoolStats.setWaiters(basicPoolStats.getWaiters() - 1);
        }
    }

    @Override
    public void getMethodReturned(final E result, final IWaitControl timeout) {
        synchronized (basicPoolStats) {
            basicPoolStats.setNotSendedBack(basicPoolStats.getNotSendedBack() + 1);
            basicPoolStats.setWaiters(basicPoolStats.getWaiters() - 1);
        }
    }

    @Override
    public void putMethodReturned(final E poolItem) {
        synchronized (basicPoolStats) {
            basicPoolStats.setNotSendedBack(basicPoolStats.getNotSendedBack() - 1);
        }
    }

    @Override
    public void removeMethodReturned(final E poolItem) {
        synchronized (basicPoolStats) {
            basicPoolStats.setNotSendedBack(basicPoolStats.getNotSendedBack() - 1);
        }
    }

    public void copyStats(final BasicPoolStats stats) {
        synchronized (basicPoolStats) {
            stats.setNotSendedBack(basicPoolStats.getNotSendedBack());
            stats.setWaiters(basicPoolStats.getWaiters());
        }
    }

}
